/*
 * Created on Jan 28, 2005
 *
 * (C) 2005 Amin Ahmad and Ahmadsoft Partners Ltd.
 */
package org.ahmadsoft.foprocessor.ui.dialogs;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.concurrent.CountDownLatch;

import org.ahmadsoft.foprocessor.FoProcessorPlugin;
import org.apache.avalon.framework.logger.Logger;
import org.apache.fop.apps.Driver;
import org.apache.fop.render.Renderer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.debug.internal.ui.preferences.IDebugPreferenceConstants;
import org.eclipse.debug.ui.IDebugUIConstants;
import org.eclipse.debug.ui.console.ConsoleColorProvider;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.ITextViewer;
import org.eclipse.jface.text.TextViewer;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.ProgressBar;
import org.eclipse.swt.widgets.Shell;
import org.xml.sax.InputSource;

/**
 *
 *
 * @author Admin
 */
public class RenderProgress extends TitleAreaDialog implements Logger {

    private Image image = null;
    private ITextViewer console;
    private Document document;
    
    private ProgressBar progressBar;
    private Label consoleTitle;
    
    private ConsoleColorProvider colorProvider = new ConsoleColorProvider();
    
    private CountDownLatch createSignal = new CountDownLatch(1);
    
    /**
     * @param parentShell
     */
    public RenderProgress(Shell parentShell) {
        super(parentShell);        
        setShellStyle(SWT.TITLE | SWT.BORDER | SWT.APPLICATION_MODAL | SWT.RESIZE);
    }
    
    protected Point getInitialSize() {
        return getShell().computeSize(460, 500, true);
    }
    
    protected Control createContents(Composite parent) {
        Control result = super.createContents(parent);
        
        createSignal.countDown();
        
        return result;
    }
    
    protected Control createDialogArea(Composite parent) {
        Composite compositeParent = (Composite)super.createDialogArea(parent);
        
        // Setup the title area
        //
        setTitleImage(getImage());
        setTitle("Rendering"); 
        setMessage("Rendering documents."); 
        
        Composite pageContainer = new Composite(compositeParent, SWT.NONE);
        GridData gd = new GridData(GridData.FILL_BOTH);
        pageContainer.setLayoutData(gd);
        pageContainer.setFont(parent.getFont());       
        
        // Document 
        //
        document = new Document();
        
        //layout for page
        //
        GridLayout layout = new GridLayout();
        layout.numColumns = 1;
        
        pageContainer.setLayout(layout);
        
        progressCaption = new Label(pageContainer, SWT.NONE);
        progressCaption.setText("Progress:");
        progressCaption.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        
        progressBar = new ProgressBar(pageContainer, SWT.SMOOTH | SWT.HORIZONTAL);
        gd = new GridData(GridData.FILL_HORIZONTAL);
        gd.heightHint = 10;
        progressBar.setLayoutData(gd);
        
        consoleTitle = new Label(pageContainer, SWT.NONE);
        consoleTitle.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        
        console = new TextViewer(pageContainer, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
        console.getTextWidget().setFont(JFaceResources.getFont(IDebugPreferenceConstants.CONSOLE_FONT));
        console.getTextWidget().setLayoutData(new GridData(GridData.FILL_BOTH));
        console.setDocument(document);
        
        // stream.setColor(fColorProvider.getColor(IDebugUIConstants.ID_STANDARD_ERROR_STREAM));
        
        
        // Build the separator line
        //
        Label separator= new Label(compositeParent, SWT.HORIZONTAL | SWT.SEPARATOR);
        separator.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        
        return compositeParent;
    }

    public Image getImage() {
        if (image == null) {
            image = FoProcessorPlugin.getDefault().getImageDescriptor("full/wizban/RenderProgress.gif").createImage();
        }
        return image;
    }
    
    public void setConsoleTitle() {
        SimpleDateFormat df = new SimpleDateFormat("MMM dd, yyyy HH:mm:ss");
        consoleTitle.setText("XSL-FO Rendering [Powered by Apache FOP] (" +df.format(new Date())+ ")");
    }
    
    public void beginRendering(Renderer renderer, String extension, IStructuredSelection resources) {
        try {
            createSignal.await();
        } catch (InterruptedException e) {
            error(null, e);
        }
        
        // Create the Progress Monitor
        //
        ProgressMonitor monitor = new ProgressMonitor();
        monitor.beginTask("Initializing", 3);
        
        // Set the titles and the button states
        //
        setConsoleTitle();
        
        Button okButton = getButton(IDialogConstants.OK_ID);
        if (okButton != null) {
            okButton.setEnabled(false);
        }
        
        Button cnButton = getButton(IDialogConstants.CANCEL_ID);
        if (cnButton != null) {
            cnButton.setEnabled(true);
        }

        // Setup my driver.
        //
        Driver driver = new Driver();
        
        // Setup logger
        // 
        driver.setLogger(this);
        
        // Setup the renderer.
        //
        driver.setRenderer(renderer);
        monitor.worked(1);
        
        // Determine information about the files.
        //
        long totalByteSize = 0;
        for (Iterator i=resources.iterator(); i.hasNext(); ) {
            IResource resource = (IResource) i.next();
            if (resource instanceof IFile) {
                IFile inputFile  = (IFile) resource;
                File file = inputFile.getRawLocation().toFile();
                
                if (file.exists()) {
                    totalByteSize += file.length();
                }
            }
        }

        monitor.beginTask("Rendering", totalByteSize > Integer.MAX_VALUE ? Integer.MAX_VALUE: (int) totalByteSize);
        // Rendition loop
        //
        for (Iterator i=resources.iterator(); i.hasNext(); ) {
            IResource resource = (IResource) i.next();
            if (resource instanceof IFile) {
                IFile inputFile  = (IFile) resource;
                IPath inputPath  = inputFile.getRawLocation();
                IPath outputPath = inputPath.removeFileExtension().addFileExtension(extension);
                
                //IFile outputFile = resource.getProject().getFile(outputPath);
                
                OutputStream out = null;
                InputStream  in  = null;
                
                File file = outputPath.toFile();
                try {
                    out = new FileOutputStream(file);
                    driver.setOutputStream(out);
                    
                    in = inputFile.getContents();
                    driver.setInputSource(new InputSource(in));
                    
                    monitor.subTask(outputPath.lastSegment());
                    driver.run();
                    monitor.worked((int)inputFile.getRawLocation().toFile().length());
                } catch (Exception e) {
                    error(null, e);
                    continue;
                } finally {
                    try {
                        if (out != null) out.close();
                        if (in != null) in.close();
                    } catch (IOException e) {
                        error(null, e);
                    }
                    
                    try {
                        inputFile.getParent().refreshLocal(1, new NullProgressMonitor());
                    } catch (CoreException e) {
                        error(null, e);
                    }
                }
            }
        }
        
        monitor.done();
        
        okButton = getButton(IDialogConstants.OK_ID);
        if (okButton != null) {
            okButton.setEnabled(true);
            okButton.setFocus();
        }
        
        cnButton = getButton(IDialogConstants.CANCEL_ID);
        if (cnButton != null) {
            cnButton.setEnabled(false);
        }
    }

    
    /**
     * @inheritDoc
     */
    public boolean close() {
        getImage().dispose();
        return super.close();
    }
    
    // Logging Methods --------------------------------------------------------

    private static final String nl = System.getProperty("line.separator");
    private Label progressCaption;
    
    private String toString(Throwable t) {
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        PrintStream ps = new PrintStream(bos);
        t.printStackTrace(ps);
        ps.close();
        
        return new String(bos.toByteArray());
    }
    
    private void append(String s, Color color) {
        if (s == null) {
            return;
        }
        try {
            StyleRange styleRange = new StyleRange();
            styleRange.start = document.getLength() - 1;
            styleRange.length = s.length() + nl.length();
            styleRange.foreground = color;
            
            document.replace(document.getLength(), 0, s);
            document.replace(document.getLength(), 0, nl);
            
            console.getTextWidget().setStyleRange(styleRange);
        } catch (BadLocationException e) {
            e.printStackTrace();
        }
    }
    
    /**
     * @inheritdoc
     */
    public void debug(String arg0) {
        append(arg0, colorProvider.getColor(IDebugUIConstants.ID_STANDARD_OUTPUT_STREAM));
    }

    /**
     * @inheritdoc
     */
    public void debug(String arg0, Throwable arg1) {
        append(arg0, colorProvider.getColor(IDebugUIConstants.ID_STANDARD_OUTPUT_STREAM));
        append(toString(arg1), colorProvider.getColor(IDebugUIConstants.ID_STANDARD_OUTPUT_STREAM));
    }

    /**
     * @inheritdoc
     */
    public boolean isDebugEnabled() {
        return true;
    }

    /**
     * @inheritdoc
     */
    public void info(String arg0) {
        append(arg0, colorProvider.getColor(IDebugUIConstants.ID_STANDARD_OUTPUT_STREAM));
    }

    /**
     * @inheritdoc
     */
    public void info(String arg0, Throwable arg1) {
        append(arg0, colorProvider.getColor(IDebugUIConstants.ID_STANDARD_OUTPUT_STREAM));
        append(toString(arg1), colorProvider.getColor(IDebugUIConstants.ID_STANDARD_OUTPUT_STREAM));
    }

    /**
     * @inheritdoc
     */
    public boolean isInfoEnabled() {
        return true;
    }

    /**
     * @inheritdoc
     */
    public void warn(String arg0) {
        append(arg0, colorProvider.getColor(IDebugUIConstants.ID_STANDARD_OUTPUT_STREAM));
    }

    /**
     * @inheritdoc
     */
    public void warn(String arg0, Throwable arg1) {
        append(arg0, colorProvider.getColor(IDebugUIConstants.ID_STANDARD_OUTPUT_STREAM));
        append(toString(arg1), colorProvider.getColor(IDebugUIConstants.ID_STANDARD_OUTPUT_STREAM));
    }

    /**
     * @inheritdoc
     */
    public boolean isWarnEnabled() {
        return true;
    }

    /**
     * @inheritdoc
     */
    public void error(String arg0) {
        append(arg0, colorProvider.getColor(IDebugUIConstants.ID_STANDARD_ERROR_STREAM));
    }

    /**
     * @inheritdoc
     */
    public void error(String arg0, Throwable arg1) {
        append(arg0, colorProvider.getColor(IDebugUIConstants.ID_STANDARD_ERROR_STREAM));
        append(toString(arg1), colorProvider.getColor(IDebugUIConstants.ID_STANDARD_ERROR_STREAM));
        
    }

    /**
     * @inheritdoc
     */
    public boolean isErrorEnabled() {
        return true;
    }

    /**
     * @inheritdoc
     */
    public void fatalError(String arg0) {
        append(arg0, colorProvider.getColor(IDebugUIConstants.ID_STANDARD_ERROR_STREAM));
    }

    /**
     * @inheritdoc
     */
    public void fatalError(String arg0, Throwable arg1) {
        append(arg0, colorProvider.getColor(IDebugUIConstants.ID_STANDARD_ERROR_STREAM));
        append(toString(arg1), colorProvider.getColor(IDebugUIConstants.ID_STANDARD_ERROR_STREAM));
    }

    /**
     * @inheritdoc
     */
    public boolean isFatalErrorEnabled() {
        return true;
    }

    /**
     * @inheritdoc
     */
    public Logger getChildLogger(String arg0) {
        return null;
    }
    
    // Progress Monitor -------------------------------------------------------
    
    public class ProgressMonitor implements IProgressMonitor {

        private String name;
        private boolean canceled;

        /**
         * @inheritdoc
         */
        public void beginTask(String name, int totalWork) {
            progressBar.setMinimum(0);
            progressBar.setSelection(0);
            progressBar.setMaximum(totalWork);

            progressCaption.setText("Progress: " + name);
            this.name = name;
        }

        /**
         * @inheritdoc
         */
        public void done() {
            progressBar.setSelection(progressBar.getMaximum());
            progressCaption.setText("Progress: Done");
        }

        /**
         * @inheritdoc
         */
        public void internalWorked(double work) {
            throw new UnsupportedOperationException("internalWorked not supported");
        }

        /**
         * @inheritdoc
         */
        public boolean isCanceled() {
            return canceled;
        }

        /**
         * @inheritdoc
         */
        public void setCanceled(boolean value) {
            this.canceled = value;
            if (value) {
                progressCaption.setText("Progress: " + this.name + "... Requesting cancelation");
            }
        }

        /**
         * @inheritdoc
         */
        public void setTaskName(String name) {
            this.name = name;
        }

        /**
         * @inheritdoc
         */
        public void subTask(String name) {
            progressCaption.setText("Progress: " + this.name + "... " + name);
        }

        /**
         * @inheritdoc
         */
        public void worked(int work) {
            progressBar.setSelection(progressBar.getSelection() + work);
        }
        
    }
}
